/* AC Switch Test-Program
   Program By GucciTech - Chojiro Higashiguchi
   Date 2008/06/28
   Program Name 'AC_Switch_V2.C'
   PIC MPU PIC16F876A-I/SP
   LCD = SD1602HULB-XA
   
   I/O Pin Layout
      PORT-A4     Up SW
      PORT-A5     Down SW

      PORT-B0     AC SYNC Pulse Input
      PORT-B5     AC SSR-SW

      PORTC-2     LCD RS
      PORTC-3     LCD E
      PORTC-4     LCD DB4
      PORTC-5     LCD DB5
      PORTC-6     LCD DB6
      PORTC-7     LCD DB7
*/
#include "AC_Switch_V2.h"
#include "LCD_G1.h"

static   unsigned char  FLAG1 = 0;
static   unsigned int   FLAG2 = 0;
static   unsigned int   counter1 = 0;  // 1sec for Counter
static   unsigned int   counter2 = 0;  // Frequency Counter
static   unsigned int   Frequency = 0;  // Frequency Counter

//***** 1/f Interrupts Cycle Time *****//
#int_EXT
void  EXT_isr(void)
{
   //***** AC Pulse 'ZERO' Check! *****//
   if (FLAG1 == 0){
      FLAG1 = 1;
   }
   //***** AC Cycle Counter Check! *****//
   counter2 ++;                        // Frequency Counter + 1
   if (FLAG2 == 1)
   {
      Frequency = counter2;
      counter2 =  0;
      FLAG2 = 0;
   }
}

//***** 1sec Timer Interrupts Cycle about 0.1sec *****//

#int_TIMER1
void  TIMER1_isr(void) 
{
   set_timer1(0x0bdc);                 // 100ms ReSetting
   counter1 ++;                        // 1sec for Counter + 1
   if (counter1 > 9)
   {
      counter1 = 0;
      FLAG2 = 1;
   }
}

//***** Para Mater Initialize *****//
short SSR_ON = TRUE;
short SSR_OFF = FALSE;
int   ON_cnt = 0;
int   OFF_cnt = 10;
int   cnt = 0;
int   Duty_cnt = 0;

void main()
{
   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_OFF);
   setup_spi(SPI_SS_DISABLED);
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
      set_timer1(0x0bdc);                 // 100ms \setting
   setup_timer_2(T2_DISABLED,0,1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);

   // TODO: USER CODE!!

   output_bit(AC_SSR_sw,SSR_OFF);   // AC SSR-SW 'OFF'
   LCD_init();                      // LCD Initialize
   LCD_clear();                     // LCD Display Clear
   Duty_cnt = ON_cnt * 10;

   LCD_control(0x80);               // move to line 1
   printf(LCD_display, "AC Controller V1");
   delay_ms(200);                   // Delay 200ms

   //***** Interrupts Enable *****//
   enable_interrupts(INT_EXT);
   enable_interrupts(INT_TIMER1);
   enable_interrupts(GLOBAL);

   //***** Frequency LCD Disp *****//
//   delay_ms(1100);                   // Delay 1.2sec
//   LCD_control(0xc0);                // move to line 2
//   printf(LCD_display,"F=%2uHz Duty=%2u%%",Frequency,LCD_cnt);
//   delay_ms(100);                   // Delay 200ms

   while(TRUE)
   {
      LCD_control(0xc0);               // move to line 2
      printf(LCD_display,"F=%2uHz Duty=%3u%%",Frequency,Duty_cnt);
      if (FLAG1 == 1)
      {
         for (cnt = 0; cnt < ON_cnt; cnt++)
         {
            while(FLAG1 == 0)
               ;
            FLAG1 = 0;
            output_bit(AC_SSR_sw,SSR_ON);    // AC SSR-SW 'ON'
         }
         for (cnt = 0; cnt < OFF_cnt; cnt++)
         {
            while(FLAG1 == 0)
               ;
            FLAG1 = 0;
            output_bit(AC_SSR_sw,SSR_OFF);   // AC SSR-SW 'OFF'
         } 
      }
      switch (input_a() & 0b00110000)
      {
         case (0b00100000):    // Up Switch Check!?
         {
            if (ON_cnt != 10)
               ON_cnt++;
               if (ON_cnt > 10)
                  ON_cnt = 0;
               OFF_cnt = 10 - ON_cnt;
               Duty_cnt = ON_cnt * 10;
            break;
         }
         case (0b00010000):    // Down Switch Check!?
         {
            if (OFF_cnt != 10)
               OFF_cnt++;
               if (OFF_cnt > 10)
                  OFF_cnt = 0;
               ON_cnt = 10 - OFF_cnt;
               Duty_cnt = ON_cnt * 10;
               break;
         }
         default : break;
      }
      LCD_control(0xc0);               // move to line 2
      printf(LCD_display,"F=%2uHz Duty=%3u%%",Frequency,Duty_cnt);
      while((input_a() & 0b00110000) != 0x30)
      {
      }
      delay_us(500);
   }
}
